
Args <- commandArgs(TRUE)
stopifnot(length(Args) == 2)
computer <- Args[1]
stopifnot(Args[1] == "DELLVISTA" ||  Args[1] == "NETBOOKUBUNTU" || Args[1] == "CLUSTER")
config.dir <- Args[2]
# input config is here. All output goes in subdirectories of this


# where this code is
if (computer=="DELLVISTA") {
  setwd("C:/AAA/Programming/Goteborg/simulation_di_tets_multigene")
  } else if  (computer=="NETBOOKUBUNTU") {
  setwd("/home/graham/simulation_di_tets_multigene")
  } else if  (computer=="CLUSTER") {
  setwd("/home/gjones/simulation_di_tets_multigene")
  }



# java classpath for calling BEAST
if (computer=="DELLVISTA") {
  java.classpath <- ".;C:\\AAA\\Programming\\Goteborg\\workspace\\beast16\\bin;C:\\AAA\\Programming\\Goteborg\\workspace\\beast16\\lib\\*"
  } else if  (computer=="NETBOOKUBUNTU") {
  java.classpath <- "'.:/home/graham/Programming/workspace/beast16/bin/:/home/graham/Programming/workspace/beast16/lib/*'" 
  } else if  (computer=="CLUSTER") {
  java.classpath <- "'.:/home/gjones/beast16/build/:/home/gjones/beast16/lib/*'"
  }   

               
# for calling Seq-Gen             
if (computer=="DELLVISTA") {
  seqgenbasecmd <- "seqgen.exe -mHKY -t3.0 -f0.3,0.2,0.2,0.3"
  } else if  (computer=="NETBOOKUBUNTU") {
  seqgenbasecmd <- "$PWD/seqgen -mHKY -t3.0 -f0.3,0.2,0.2,0.3" 
  } else if  (computer=="CLUSTER") {
  seqgenbasecmd <- "$PWD/seqgen -mHKY -t3.0 -f0.3,0.2,0.2,0.3"
  }
  



library(ape)

source("randomseeds.r")

source("AlloppSim_1readconfig2.r")
source("AlloppSim_2makemultree.r")
source("AlloppSim_3makegtrees.r")
source("AlloppSim_4makeseqfiles.r")
source("AlloppSim_5beastxml_toplevel.r")
source("AlloppSim_6beastxml_bits.r")




make.beastxml.for.one.GNT.one.replicate <- function(config, i, r) {  
  # Simulate the gene trees
  for (g in 1:config$configG[i]) {
    gt <- make.one.genetree.as.Newick(multree, config$configT[i], randomseeds[g, r])
    sink(fpath.gtree(config, i, r, g))
    cat(gt)
    sink(NULL)
    }
  
  # Simulate the sequence alignments
  for (g in 1:config$configG[i]) {  
    SeqGen.cmd <- paste(seqgenbasecmd, 
                        " -l", config$seqs$len, 
                        " -n1 -z", randomseeds[g, r], 
                        " ", fpath.gtree(config, i, r, g), 
                        " -y ", fpath.seq(config, i, r, g), 
                        sep="")
    cat("Running seqgen with command:", SeqGen.cmd, "\n")
    system(SeqGen.cmd)
    }
    
  
  # Make BEAST XML
  sink(fpath.beastxml(config, i, r))
  make.beastxml(config$network, config$configG[i],
                collect.taxa(multree), 
                fpath.seqs.fpathbase(config, i, r), 
                fpath.sampledgtrees.fpathbase(config, i, r),
                fpath.sampledmultrees(config, i, r), 
                fpath.sampledparams(config, i, r), 
                fpath.DBUGTUNE(config, i, r)
                )
  sink(NULL)
  
}


  

run.beast.for.one.GNT.one.replicate <- function(config, i, r) {
  BEAST.cmd <- paste("java -classpath", java.classpath, 
          "dr.app.beast.BeastMain -seed 42 -overwrite -working", 
          fpath.beastxml(config, i, r))
  system(BEAST.cmd)
  }  
  

run.treeannotator.for.one.GNT.one.replicate <- function(config, i, r) {
  for (g in 1:config$configG[i]) {
    TreeAnnotator.cmd <-paste("java -classpath", java.classpath, 
          "dr.app.tools.TreeAnnotator -burnin 200", 
          fpath.sampledgtree(config, i, r, g),
          fpath.mccgtree(config, i, r, g)
          )
    system(TreeAnnotator.cmd)
    }
  TreeAnnotator.cmd <-paste("java -classpath", java.classpath, 
          "dr.app.tools.TreeAnnotator -burnin 200",
          fpath.sampledmultrees(config, i, r),
          fpath.mccmultree(config, i, r)
          )      
   system(TreeAnnotator.cmd)
  }
  
  
simplify.mcc.for.one.GNT.one.replicate <- function(config, i, r) {
  for (g in 1:config$configG[i]) {
    mccmul <- read.nexus(file=fpath.mccgtree(config, i, r, g))
    #cat(scenario.name, "G =", config$configG[i], "N =", config$configN[i], 
    #       "T =", config$configT[i], "replicate", r, "gene tree", g, "\n")
    #cat(write.tree(mccmul), "\n")
    }
  
  mccmul <- read.nexus(file=fpath.mccmultree(config, i, r))
  sametopol <- all.equal(simmul1, mccmul, use.edge.length = FALSE)
  sametopol <- sametopol  ||  all.equal(simmul2, mccmul, use.edge.length = FALSE)
  cat(scenario.name, "G =", config$configG[i], "N =", config$configN[i], 
         "T =", config$configT[i], "replicate", r, "MUL-tree", sametopol, "\n")
  cat(write.tree(mccmul), "\n")
  }
 
 
mccmultree.is.simmultree <- function(config, i, r) {
  mccmul <- read.nexus(file=fpath.mccmultree(config, i, r))
  sametopol <- all.equal(simmul1, mccmul, use.edge.length = FALSE)
  sametopol <- sametopol  ||  all.equal(simmul2, mccmul, use.edge.length = FALSE)
  sametopol
  }



####################################################################
####################################################################

 
 

scenario.name <- substr(config.dir, nchar(config.dir), nchar(config.dir)) 
stopifnot(scenario.name == "A" || scenario.name == "B" || scenario.name == "C")

cat("AlloppSim running with directory", config.dir, "\n")


# this should be calculated better!
if (scenario.name == "A") {
  simmul1 <- read.tree(text="((a0:0.02,z0:0.02):0.02,(b0:0.0,z1:0.03):0.01);")  
  simmul2 <- read.tree(text="((a0:0.02,z1:0.02):0.02,(b0:0.0,z0:0.03):0.01);") 
  } else if (scenario.name == "B") {
  simmul1 <- read.tree(text="(((a0:0.02,z0:0.02):0.01,z1:0.03):0.01,b0:0.04);")    
  simmul2 <- read.tree(text="(((a0:0.02,z1:0.02):0.01,z0:0.03):0.01,b0:0.04);")    
  } else if (scenario.name == "C") {
  simmul1 <- read.tree(text="((a0:0.03,(z0:0.02,z1:0.02):0.01):0.01,b0:0.04);")
  simmul2 <- read.tree(text="((a0:0.03,(z1:0.02,z0:0.02):0.01):0.01,b0:0.04);")
  }


config <- read.config2(paste(config.dir, "/", "config.txt", sep=""))
nofGNT <- length(config$configG)


  
for (i in 1:nofGNT) {
  config$network <- fillin.indivs(config$network, config$configN[i])
  stopifnot(config$configG[i] <= dim(randomseeds)[1])
  dir.create(fpath.GNTdpath(config, i))
  
  # Make the MUL-tree
  multree <- make.multree(config$network)
  
  sink(fpath.multree(config, i))
  print.multree(multree)
  sink(NULL)
  
  # 
  R <- config$configR[i]
  stopifnot(R < dim(randomseeds)[2]) 
  for (r in 1:R) {
    make.beastxml.for.one.GNT.one.replicate(config, i, r)
    }
  }
  
  
for (i in 1:nofGNT) {
  config$network <- fillin.indivs(config$network, config$configN[i])
  R <- config$configR[i]
  stopifnot(R < dim(randomseeds)[2]) 
  for (r in 1:R) {
    run.beast.for.one.GNT.one.replicate(config, i, r)
    }
  }
  

for (i in 1:nofGNT) {
  R <- config$configR[i]
  stopifnot(R < dim(randomseeds)[2]) 
  for (r in 1:R) {
    run.treeannotator.for.one.GNT.one.replicate(config, i, r)
    }
  }
  
  
for (i in 1:nofGNT) {
  R <- config$configR[i]
  stopifnot(R < dim(randomseeds)[2]) 
  for (r in 1:R) {
    simplify.mcc.for.one.GNT.one.replicate(config, i, r)
    }
  }



for (i in 1:nofGNT) {
  R <- config$configR[i]
  nok <- 0
  for (r in 1:R) {
    if (mccmultree.is.simmultree(config, i, r)) {
      nok <- nok + 1
      }
    }
  cat(scenario.name, "G =", config$configG[i], "N =", config$configN[i], 
         "T =", config$configT[i], nok, "/", R, "\n")
  }

